import django
from django.core import mail
from django.test import TestCase
try:
    try:
        from django.utils import unittest
    except PendingDeprecationWarning:
        import unittest
except ImportError:
    import unittest  # NOQA
try:
    from django.test.utils import override_settings
except ImportError:
    def override_settings(*args, **kwargs):
        return unittest.skipIf(django.get_version().startswith('1.3'), "Django < 1.4 doesn't have override_settings")
from django.core.exceptions import ImproperlyConfigured


from emailtools import BasicEmail, HTMLEmail, MarkdownEmail


class TestBasicCBE(TestCase):
    EMAIL_ATTRS = {
        'subject': 'test email',
        'to': ['to@example.com'],
        'from_email': 'from@example.com',
        'body': 'This is a test email',
    }

    def setUp(self):
        class TestEmail(BasicEmail):
            subject = self.EMAIL_ATTRS['subject']
            to = self.EMAIL_ATTRS['to']
            from_email = self.EMAIL_ATTRS['from_email']
            body = self.EMAIL_ATTRS['body']
        self.TestEmail = TestEmail

    def create_and_send_a_message(self, **kwargs):
        email_callable = self.TestEmail.as_callable(**kwargs)
        email_callable()

    def test_mail_is_sent(self):
        self.create_and_send_a_message()
        self.assertEqual(len(mail.outbox), 1)

    def test_create_message(self):
        email_callable = self.TestEmail.as_callable()
        message = email_callable.message()

        self.assertTrue(isinstance(message, self.TestEmail.email_message_class))

    def test_to(self):
        self.create_and_send_a_message()
        message = mail.outbox[0]
        for k, v in self.EMAIL_ATTRS.iteritems():
            self.assertEqual(getattr(message, k), v)
        self.assertEqual(message.bcc, [])
        self.assertEqual(message.cc, [])

    def test_settings_override(self):
        self.create_and_send_a_message(
            to=['overridden_to@example.com'],
            cc=['overridden_cc@example.com'],
            bcc=['overridden_bcc@example.com'],
            subject='overridden_subject',
        )
        message = mail.outbox[0]
        self.assertEqual(message.to, ['overridden_to@example.com'])
        self.assertEqual(message.cc, ['overridden_cc@example.com'])
        self.assertEqual(message.bcc, ['overridden_bcc@example.com'])
        self.assertEqual(message.subject, 'overridden_subject')

    def test_improper_settings_override(self):
        with self.assertRaises(TypeError):
            self.create_and_send_a_message(not_a_class_property=True)

    def test_to_address_as_string(self):
        self.create_and_send_a_message(to='string@example.com')
        message = mail.outbox[0]
        self.assertEqual(message.to, ['string@example.com'])

    def test_missing_message_class(self):
        with self.assertRaises(ImproperlyConfigured):
            self.TestEmail.as_callable(email_message_class=None)()

    def test_missing_to(self):
        with self.assertRaises(ImproperlyConfigured):
            self.create_and_send_a_message(to=None)()

    def test_missing_from(self):
        with self.assertRaises(ImproperlyConfigured):
            self.create_and_send_a_message(from_email=None)()

    def test_missing_subject(self):
        with self.assertRaises(ImproperlyConfigured):
            self.create_and_send_a_message(subject=None)()

    def test_missing_body(self):
        with self.assertRaises(ImproperlyConfigured):
            self.create_and_send_a_message(body=None)()

    @unittest.expectedFailure
    def test_sending_kwargs(self):
        class SendingKwargsEmail(self.TestEmail):
            from_email = 'with\nnewline@gmail.com'
            fail_silently = True

        SendingKwargsEmail.as_callable(fail_silently=True)()
        with self.assertRaises(mail.BadHeaderError):
            SendingKwargsEmail.as_callable()()


class TestHTMLEmail(TestCase):
    EMAIL_ATTRS = {
        'subject': 'test email',
        'to': ['to@example.com'],
        'from_email': 'from@example.com',
        'template_name': 'tests/test_HTMLEmail_template.html',
    }

    def setUp(self):
        class TestHTMLEmail(HTMLEmail):
            subject = self.EMAIL_ATTRS['subject']
            to = self.EMAIL_ATTRS['to']
            from_email = self.EMAIL_ATTRS['from_email']
            template_name = self.EMAIL_ATTRS['template_name']

            def get_context_data(self, **kwargs):
                kwargs = super(TestHTMLEmail, self).get_context_data(**kwargs)
                kwargs.update({
                    'title': 'test title',
                    'content': 'test content',
                })
                return kwargs

        self.TestHTMLEmail = TestHTMLEmail

    def create_and_send_a_message(self, **kwargs):
        email_callable = self.TestHTMLEmail.as_callable(**kwargs)
        email_callable()

    def test_mail_is_sent(self):
        self.create_and_send_a_message()
        self.assertEqual(len(mail.outbox), 1)

    def test_mail_has_html_body(self):
        self.create_and_send_a_message()
        message = mail.outbox[0]
        self.assertTrue(message.alternatives)
        self.assertEqual(message.alternatives[0][1], 'text/html')

    @unittest.skipIf(django.get_version().startswith('1.3'), "Django < 1.4 doesn't allow assertTemplateUsed as a context manager")
    def test_template_used(self):
        with self.assertTemplateUsed(template_name=self.EMAIL_ATTRS['template_name']):
            self.create_and_send_a_message()

    def test_html_body(self):
        self.create_and_send_a_message()
        message = mail.outbox[0]
        html_body = message.alternatives[0][0]
        try:
            self.assertInHTML('<h1>test title</h1>', html_body)
            self.assertInHTML('<p>test content</p>', html_body)
        except AttributeError:  # support for < django 1.5
            self.assertIn('<h1>test title</h1>', html_body)
            self.assertIn('<p>test content</p>', html_body)

    def test_plain_body(self):
        self.create_and_send_a_message()
        message = mail.outbox[0]
        self.assertIn('test title', message.body)
        self.assertIn('test content', message.body)
        self.assertNotIn('<h1>', message.body)
        self.assertNotIn('<p>', message.body)


class TestMarkdownEmail(TestHTMLEmail):
    EMAIL_ATTRS = {
        'subject': 'test email',
        'to': ['to@example.com'],
        'from_email': 'from@example.com',
        'template_name': 'tests/test_MarkdownEmail_template.md',
    }

    def setUp(self):
        class TestMarkdownEmail(MarkdownEmail):
            layout_template = 'mail/base.html'
            subject = self.EMAIL_ATTRS['subject']
            to = self.EMAIL_ATTRS['to']
            from_email = self.EMAIL_ATTRS['from_email']
            template_name = self.EMAIL_ATTRS['template_name']

            def get_context_data(self, **kwargs):
                kwargs = super(TestMarkdownEmail, self).get_context_data(**kwargs)
                kwargs.update({
                    'title': 'test title',
                    'content': 'test content',
                })
                return kwargs

        self.TestMarkdownEmail = TestMarkdownEmail

    def create_and_send_a_message(self, **kwargs):
        email_callable = self.TestMarkdownEmail.as_callable(**kwargs)
        email_callable()

    @override_settings(EMAIL_LAYOUT=None)
    def test_missing_base_layout(self):
        self.create_and_send_a_message()
        with self.assertRaises(ImproperlyConfigured):
            self.create_and_send_a_message(layout_template=None)



from functools import update_wrapper

from django.utils.decorators import classonlymethod
from django.core.exceptions import ImproperlyConfigured


class BaseEmail(object):
    """
    Simple base class for all class-based emails.  Implements the basic
    structure for constructing an email message and sending it, along with the
    `as_callable` method logic.
    """
    email_message_class = None

    def get_email_message_kwargs(self, **kwargs):
        return kwargs

    def get_email_message_class(self):
        if self.email_message_class is None:
            raise ImproperlyConfigured('No `email_message_class` provided')
        return self.email_message_class

    def get_email_message(self):
        return self.get_email_message_class()(**self.get_email_message_kwargs())

    def get_send_kwargs(self, **kwargs):
        return kwargs

    def send(self):
        self.get_email_message().send(**self.get_send_kwargs())

    def __init__(self, **kwargs):
        for key, value in kwargs.iteritems():
            setattr(self, key, value)

    @classonlymethod
    def as_callable(cls, **initkwargs):
        for key in initkwargs:
            if not hasattr(cls, key):
                raise TypeError("{0}() received an invalid keyword {1!r}. "
                                "as_callable only accepts arguments that are "
                                "already attributes of the "
                                "class.".format(cls.__name__, key))

        def callable(*args, **kwargs):
            self = cls(**initkwargs)
            self.args = args
            self.kwargs = kwargs
            return self.send()

        def message(*args, **kwargs):
            self = cls(**initkwargs)
            self.args = args
            self.kwargs = kwargs
            return self.get_email_message()

        callable.message = message

        update_wrapper(callable, cls, updated=())
        update_wrapper(callable.message, cls, updated=())
        return callable



import markdown

from django.template import loader
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.conf import settings
from django.utils.html import strip_tags
from django.core.exceptions import ImproperlyConfigured
from django.utils.safestring import mark_safe

from .base import BaseEmail
from .mixins import TemplateEmailMixin


class BasicEmail(BaseEmail):
    """
    Class-based email based around `django.core.email.EmailMessage`
    """
    email_message_class = EmailMessage
    subject = None
    to = None
    cc = None
    bcc = None
    from_email = None
    body = None
    connection = None
    attachments = None
    headers = None
    fail_silently = False

    def get_email_message_kwargs(self, **kwargs):
        kwargs = super(BasicEmail, self).get_email_message_kwargs()
        kwargs.update({
            'subject': self.get_subject(),
            'body': self.get_body(),
            'from_email': self.get_from_email(),
            'to': self.get_to(),
            'cc': self.get_cc(),
            'bcc': self.get_bcc(),
            'connection': self.get_connection(),
        })
        return kwargs

    def get_to(self):
        if self.to is None:
            raise ImproperlyConfigured('No `to` provided')
        if isinstance(self.to, basestring):
            return [self.to]
        return self.to

    def get_cc(self):
        return self.cc or tuple()

    def get_bcc(self):
        return self.bcc or tuple()

    def get_attachments(self):
        return self.attachments or tuple()

    def get_headers(self):
        return self.headers or {}

    def get_fail_silently(self):
        return self.fail_silently

    def get_send_kwargs(self, **kwargs):
        kwargs = super(BasicEmail, self).get_send_kwargs(**kwargs)
        kwargs.setdefault('fail_silently', self.get_fail_silently())
        return kwargs

    def get_from_email(self):
        if self.from_email is None:
            raise ImproperlyConfigured('No `from_email` provided')
        return self.from_email

    def get_subject(self):
        if self.subject is None:
            raise ImproperlyConfigured('No `subject` provided')
        return self.subject

    def get_body(self):
        if self.body is None:
            raise ImproperlyConfigured('No `body` provided')
        return self.body

    def get_connection(self):
        return self.connection


class HTMLEmail(TemplateEmailMixin, BasicEmail):
    """
    Sends an HTML email.
    """
    email_message_class = EmailMultiAlternatives

    def get_email_message(self):
        message = super(HTMLEmail, self).get_email_message()
        message.attach_alternative(self.get_rendered_template(), "text/html")
        return message

    def get_body(self):
        return strip_tags(super(HTMLEmail, self).get_body())


class MarkdownEmail(HTMLEmail):
    """
    Renders a markdown template into an HTML email.
    """
    layout_template = None
    template_name = None

    def get_layout_template(self):
        if self.layout_template is None:
            if getattr(settings, 'EMAIL_LAYOUT', None) is not None:
                return settings.EMAIL_LAYOUT
            raise ImproperlyConfigured('layout was not defined by settings.EMAIL_LAYOUT and none was provided')
        return [self.layout_template]

    def get_layout_context_data(self, **kwargs):
        return kwargs

    def get_rendered_template(self):
        md = super(MarkdownEmail, self).get_rendered_template()
        return loader.render_to_string(
            self.get_layout_template(),
            self.get_layout_context_data(content=mark_safe(markdown.markdown(md, ['extra']))),
        )



from django.contrib.auth.tokens import default_token_generator
from django.contrib.sites.models import Site
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.template import loader
from django.utils.http import int_to_base36


class TemplateEmailMixin(object):
    """
    Mixin for using template rendering for the email message body.
    """
    template_name = None

    def get_template_names(self):
        if self.template_name is None:
            raise ImproperlyConfigured('No `template_name` provided')
        return [self.template_name]

    def get_context_data(self, **kwargs):
        return kwargs

    def get_rendered_template(self):
        return loader.render_to_string(
            self.get_template_names(),
            self.get_context_data(),
        )

    def get_body(self):
        return self.get_rendered_template()


class BuildAbsoluteURIMixin(object):
    """
    Mixin which provides methods for constructing absolute uris.
    """
    protocol = 'http'

    def get_domain(self):
        return Site.objects.get_current().domain

    def get_protocol(self):
        return self.protocol

    def reverse_absolute_uri(self, view_name, args=None, kwargs=None):
        location = reverse(view_name, args=args, kwargs=kwargs)
        return self.build_absolute_uri(location)

    def build_absolute_uri(self, location):
        current_uri = '{protocol}://{domain}{location}'.format(
            protocol=self.get_protocol(),
            domain=self.get_domain(),
            location=location,
        )
        return current_uri


class UserTokenEmailMixin(BuildAbsoluteURIMixin):
    """
    Mixin which provides methods for generating token based links.
    """
    UID_KWARG = 'uidb36'
    TOKEN_KWARG = 'token'

    token_generator = default_token_generator

    def get_user(self):
        return self.args[0]

    def generate_token(self, user):
        return self.token_generator.make_token(user)

    def get_uid(self, user):
        return int_to_base36(user.pk)

    def reverse_token_url(self, view_name, args=None, kwargs={}):
        kwargs.setdefault(self.UID_KWARG, self.get_uid(self.get_user()))
        kwargs.setdefault(self.TOKEN_KWARG, self.generate_token(self.get_user()))
        return self.reverse_absolute_uri(view_name, args=args, kwargs=kwargs)



#!/usr/bin/env python
from setuptools import setup, find_packages
import subprocess
import os

__doc__ = """
App for Django featuring class based email sending
"""


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


install_requires = [
    'Django>=1.3',
    'markdown',
]

version = (0, 1, 1, 'final')


def get_version():
    number = '.'.join(map(str, version[:3]))
    stage = version[3]
    if stage == 'final':
        return number
    elif stage == 'alpha':
        process = subprocess.Popen('git rev-parse HEAD'.split(), stdout=subprocess.PIPE)
        stdout, stderr = process.communicate()
        return number + '-' + stdout.strip()[:8]

setup(
    name='django-emailtools',
    version=get_version(),
    description=__doc__,
    long_description=read('README.rst'),
    packages=[package for package in find_packages() if package.startswith('emailtools')],
    url="https://django-emailtools.readthedocs.org/en/latest/",
    author="Fusionbox",
    author_email='programmers@fusionbox.com',
    install_requires=install_requires,
    zip_safe=False,
    include_package_data=True,
)
